/*
    libhidefile -- hide some entries from directory listings
    Copyright (C) 2003  Egmont Koblinger <egmont@uhulinux.hu>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
*/

#define _GNU_SOURCE
#include <stdlib.h>
#include <dlfcn.h>
#include <stdio.h>
#include <dirent.h>
#include <string.h>

static char *hidefileenv;
static char *hidefilestr;

static struct dirent *(*next_readdir)(DIR *dir);
static struct dirent64 *(*next_readdir64)(DIR *dir);
static int (*next_readdir_r)(DIR *dir, struct dirent *result, struct dirent **resultp);
static int (*next_readdir64_r)(DIR *dir, struct dirent64 *result, struct dirent64 **resultp);

static int do_wrap = 0;

void hidefile_init (void) __attribute((constructor));
void hidefile_init (void)
{
	next_readdir     = dlsym(RTLD_NEXT, "readdir");
	next_readdir64   = dlsym(RTLD_NEXT, "readdir64");
	next_readdir_r   = dlsym(RTLD_NEXT, "readdir_r");
	next_readdir64_r = dlsym(RTLD_NEXT, "readdir64_r");
	if ((hidefileenv = getenv("HIDEFILE")) != NULL) {
		hidefilestr = malloc(strlen(hidefileenv) + 3);
		sprintf(hidefilestr, "/%s/", hidefileenv);
		do_wrap = 1;
	}
}

struct dirent *readdir (DIR *dir)
{
	char dirstr[NAME_MAX + 3];
	struct dirent *dirent;
	if (!do_wrap) return next_readdir(dir);
	while (1) {
		dirent = next_readdir(dir);
		if (dirent == NULL) return NULL;
		sprintf(dirstr, "/%s/", dirent->d_name);
		if (strstr(hidefilestr, dirstr) == NULL) return dirent;
	}
}

struct dirent64 *readdir64 (DIR *dir)
{
	char dirstr[NAME_MAX + 3];
	struct dirent64 *dirent;
	if (!do_wrap) return next_readdir64(dir);
	while (1) {
		dirent = next_readdir64(dir);
		if (dirent == NULL) return NULL;
		sprintf(dirstr, "/%s/", dirent->d_name);
		if (strstr(hidefilestr, dirstr) == NULL) return dirent;
	}
}

int readdir_r (DIR *dir, struct dirent *result, struct dirent **resultp)
{
	char dirstr[NAME_MAX + 3];
	int retval;
	if (!do_wrap) return next_readdir_r(dir, result, resultp);
	while (1) {
		retval = next_readdir_r(dir, result, resultp);
		if (retval < 0) return -1;
		sprintf(dirstr, "/%s/", result->d_name);
		if (strstr(hidefilestr, dirstr) == NULL) return 0;
	}
}

int readdir64_r (DIR *dir, struct dirent64 *result, struct dirent64 **resultp)
{
	char dirstr[NAME_MAX + 3];
	int retval;
	if (!do_wrap) return next_readdir64_r(dir, result, resultp);
	while (1) {
		retval = next_readdir64_r(dir, result, resultp);
		if (retval < 0) return -1;
		sprintf(dirstr, "/%s/", result->d_name);
		if (strstr(hidefilestr, dirstr) == NULL) return 0;
	}
}

